/*
 * Copyright (C) 2017-2023 Dominique Dhoutaut, Thierry Arrabal, Eugen Dedu.
 *
 * This file is part of BitSimulator.
 *
 * BitSimulator is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BitSimulator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BitSimulator.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef AGENTS_RAYTRACING_ROUTING_AGENT_H_
#define AGENTS_RAYTRACING_ROUTING_AGENT_H_

#include <random>
#include "routing-agent.h"


//==============================================================================
//
//          RayTracingRoutingAgent  (class)
//
//==============================================================================

class RayTracingRoutingAgent : public RoutingAgent {
protected:
  static mt19937_64 *backoffrng;
  // map<int,int> alreadySeenFlows;
  int datacounter[1];  // number of data packets received
  simulationTime_t timefirst[1];  // time when 1st packet was received
  simulationTime_t timesecond[1];  // time when 2nd packet was received
  bool unscheduled[1];
  bool ctfReceived[1];  // true if a CTF packet has been received AFTER two data packets received, no matter CTF's nodeid
  bool packetForwarded[1];
  bool rtfSent[1];
  int rtfSender[1];
  PacketPtr packetSave[1];

  virtual void processSendEvent (PacketPtr packet);
  using RayTracingSendEvent = CallMethodArgEvent<RayTracingRoutingAgent,
                                                 PacketPtr,
                                                 &RayTracingRoutingAgent::processSendEvent,
                                                 EventType::GENERIC>;

public:
  RayTracingRoutingAgent(Node *_hostNode);
  virtual ~RayTracingRoutingAgent();

  static void initializeAgent();
  virtual void receivePacketFromApplication(PacketPtr packet);
  virtual void receivePacketFromNetwork(PacketPtr packet);
};

#endif /* AGENTS_RAYTRACING_ROUTING_AGENT_H_ */
